package org.jeecg.modules.adweb.subscription.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.subscription.entity.UserPlanSubscription;
import org.jeecg.modules.adweb.subscription.service.IUserPlanSubscriptionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.Arrays;

/**
 * @Description: AdWeb套餐订购
 * @Author: jeecg-boot
 * @Date: 2024-10-15
 * @Version: V1.0
 */
@Tag(name = "AdWeb套餐订购")
@RestController
@RequestMapping("/adweb/userPlanSubscription")
@Slf4j
public class UserPlanSubscriptionController
        extends JeecgController<UserPlanSubscription, IUserPlanSubscriptionService> {
    @Autowired private IUserPlanSubscriptionService userPlanSubscriptionService;

    /**
     * 分页列表查询
     *
     * @param userPlanSubscription
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    // @AutoLog(value = "AdWeb套餐订购-分页列表查询")
    @Operation(summary = "AdWeb套餐订购-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserPlanSubscription>> queryPageList(
            UserPlanSubscription userPlanSubscription,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            HttpServletRequest req) {
        QueryWrapper<UserPlanSubscription> queryWrapper =
                QueryGenerator.initQueryWrapper(userPlanSubscription, req.getParameterMap());
        Page<UserPlanSubscription> page = new Page<UserPlanSubscription>(pageNo, pageSize);
        IPage<UserPlanSubscription> pageList = userPlanSubscriptionService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param userPlanSubscription
     * @return
     */
    @AutoLog(value = "AdWeb套餐订购-添加")
    @Operation(summary = "AdWeb套餐订购-添加")
    @PreAuthorize("@jps.requiresPermissions('adweb:user_plan_subscription:add')")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody UserPlanSubscription userPlanSubscription) {
        userPlanSubscriptionService.save(userPlanSubscription);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param userPlanSubscription
     * @return
     */
    @AutoLog(value = "AdWeb套餐订购-编辑")
    @Operation(summary = "AdWeb套餐订购-编辑")
    @PreAuthorize("@jps.requiresPermissions('adweb:user_plan_subscription:edit')")
    @RequestMapping(
            value = "/edit",
            method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody UserPlanSubscription userPlanSubscription) {
        userPlanSubscriptionService.updateById(userPlanSubscription);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "AdWeb套餐订购-通过id删除")
    @Operation(summary = "AdWeb套餐订购-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('adweb:user_plan_subscription:delete')")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        userPlanSubscriptionService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "AdWeb套餐订购-批量删除")
    @Operation(summary = "AdWeb套餐订购-批量删除")
    @PreAuthorize("@jps.requiresPermissions('adweb:user_plan_subscription:deleteBatch')")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.userPlanSubscriptionService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    // @AutoLog(value = "AdWeb套餐订购-通过id查询")
    @Operation(summary = "AdWeb套餐订购-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserPlanSubscription> queryById(
            @RequestParam(name = "id", required = true) String id) {
        UserPlanSubscription userPlanSubscription = userPlanSubscriptionService.getById(id);
        if (userPlanSubscription == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(userPlanSubscription);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param userPlanSubscription
     */
    @PreAuthorize("@jps.requiresPermissions('adweb:user_plan_subscription:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(
            HttpServletRequest request, UserPlanSubscription userPlanSubscription) {
        return super.exportXls(
                request, userPlanSubscription, UserPlanSubscription.class, "AdWeb套餐订购");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PreAuthorize("@jps.requiresPermissions('adweb:user_plan_subscription:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, UserPlanSubscription.class);
    }
}
